precision highp float;
 varying highp vec2 textureCoordinate;

 uniform sampler2D inputImageTexture;
 uniform sampler2D inputImageTexture2; //map

 uniform float inputImageTextureHeight;
 uniform float inputImageTextureWidth;

 uniform float p1;

 vec4 sharpenImage()
 {
     vec2 leftTextureCoordinate;
     vec2 rightTextureCoordinate;

     vec2 topTextureCoordinate;
     vec2 topLeftTextureCoordinate;
     vec2 topRightTextureCoordinate;

     vec2 bottomTextureCoordinate;
     vec2 bottomLeftTextureCoordinate;
     vec2 bottomRightTextureCoordinate;

     mat3 convolutionMatrix = mat3(0.0751, 0.1238, 0.0751, 0.1238, 0.2042, 0.1238, 0.0751, 0.1238, 0.0751);

     float texelWidth = 1.0 / inputImageTextureWidth;
     float texelHeight = 1.0 / inputImageTextureHeight;

     vec2 widthStep = vec2(texelWidth, 0.0);
     vec2 heightStep = vec2(0.0, texelHeight);
     vec2 widthHeightStep = vec2(texelWidth, texelHeight);
     vec2 widthNegativeHeightStep = vec2(texelWidth, -texelHeight);

     leftTextureCoordinate = textureCoordinate.xy - widthStep;
     rightTextureCoordinate = textureCoordinate.xy + widthStep;

     topTextureCoordinate = textureCoordinate.xy - heightStep;
     topLeftTextureCoordinate = textureCoordinate.xy - widthHeightStep;
     topRightTextureCoordinate = textureCoordinate.xy + widthNegativeHeightStep;

     bottomTextureCoordinate = textureCoordinate.xy + heightStep;
     bottomLeftTextureCoordinate = textureCoordinate.xy - widthNegativeHeightStep;
     bottomRightTextureCoordinate = textureCoordinate.xy + widthHeightStep;

     vec3 bottomColor = texture2D(inputImageTexture, bottomTextureCoordinate).rgb;
     vec3 bottomLeftColor = texture2D(inputImageTexture, bottomLeftTextureCoordinate).rgb;
     vec3 bottomRightColor = texture2D(inputImageTexture, bottomRightTextureCoordinate).rgb;
     vec4 centerColor = texture2D(inputImageTexture, textureCoordinate);
     vec3 leftColor = texture2D(inputImageTexture, leftTextureCoordinate).rgb;
     vec3 rightColor = texture2D(inputImageTexture, rightTextureCoordinate).rgb;
     vec3 topColor = texture2D(inputImageTexture, topTextureCoordinate).rgb;
     vec3 topRightColor = texture2D(inputImageTexture, topRightTextureCoordinate).rgb;
     vec3 topLeftColor = texture2D(inputImageTexture, topLeftTextureCoordinate).rgb;

     vec3 blurColor = topLeftColor * convolutionMatrix[0][0] + topColor * convolutionMatrix[0][1] + topRightColor * convolutionMatrix[0][2];
     blurColor += leftColor * convolutionMatrix[1][0] + centerColor.rgb * convolutionMatrix[1][1] + rightColor * convolutionMatrix[1][2];
     blurColor += bottomLeftColor * convolutionMatrix[2][0] + bottomColor * convolutionMatrix[2][1] + bottomRightColor * convolutionMatrix[2][2];

     float gaussianWeightTotal = convolutionMatrix[0][0] + convolutionMatrix[0][1] + convolutionMatrix[0][2] +
     convolutionMatrix[1][0] + convolutionMatrix[1][1] + convolutionMatrix[1][2] +
     convolutionMatrix[2][0] + convolutionMatrix[2][1] + convolutionMatrix[2][2];
     blurColor = blurColor / gaussianWeightTotal;

     vec3 sharpColor = 1.4 * centerColor.rgb - 0.4 * blurColor;

     return vec4(sharpColor, centerColor.a);
 }

 void main()
 {
     vec4 sharpColor = sharpenImage();
     vec4 textureColor = texture2D(inputImageTexture2, textureCoordinate);

     vec4 result = textureColor;
     result.rgb = mix(sharpColor.rgb ,textureColor.rgb, p1);

     gl_FragColor = result;
 }