 precision highp float;

 varying highp vec2 textureCoordinate;

 uniform sampler2D inputImageTexture;
 uniform sampler2D inputImageTexture2; //earlyBirdCurves
 uniform sampler2D inputImageTexture3; //earlyBirdOverlay
 uniform sampler2D inputImageTexture4; //vig
 uniform sampler2D inputImageTexture5; //earlyBirdBlowout
 uniform sampler2D inputImageTexture6; //earlyBirdMap

 const mat3 saturate = mat3(
                            1.210300,
                            -0.089700,
                            -0.091000,
                            -0.176100,
                            1.123900,
                            -0.177400,
                            -0.034200,
                            -0.034200,
                            1.265800);
 const vec3 rgbPrime = vec3(0.25098, 0.14640522, 0.0);
 const vec3 desaturate = vec3(.3, .59, .11);

  uniform float inputImageTextureHeight;
  uniform float inputImageTextureWidth;

 vec4 sharpenImage()
 {
     vec2 leftTextureCoordinate;
     vec2 rightTextureCoordinate;

     vec2 topTextureCoordinate;
     vec2 topLeftTextureCoordinate;
     vec2 topRightTextureCoordinate;

     vec2 bottomTextureCoordinate;
     vec2 bottomLeftTextureCoordinate;
     vec2 bottomRightTextureCoordinate;

     mat3 convolutionMatrix = mat3(0.0751, 0.1238, 0.0751, 0.1238, 0.2042, 0.1238, 0.0751, 0.1238, 0.0751);

     float texelWidth = 1.0 / inputImageTextureWidth;
     float texelHeight = 1.0 / inputImageTextureHeight;

     vec2 widthStep = vec2(texelWidth, 0.0);
     vec2 heightStep = vec2(0.0, texelHeight);
     vec2 widthHeightStep = vec2(texelWidth, texelHeight);
     vec2 widthNegativeHeightStep = vec2(texelWidth, -texelHeight);

     leftTextureCoordinate = textureCoordinate.xy - widthStep;
     rightTextureCoordinate = textureCoordinate.xy + widthStep;

     topTextureCoordinate = textureCoordinate.xy - heightStep;
     topLeftTextureCoordinate = textureCoordinate.xy - widthHeightStep;
     topRightTextureCoordinate = textureCoordinate.xy + widthNegativeHeightStep;

     bottomTextureCoordinate = textureCoordinate.xy + heightStep;
     bottomLeftTextureCoordinate = textureCoordinate.xy - widthNegativeHeightStep;
     bottomRightTextureCoordinate = textureCoordinate.xy + widthHeightStep;

     vec3 bottomColor = texture2D(inputImageTexture, bottomTextureCoordinate).rgb;
     vec3 bottomLeftColor = texture2D(inputImageTexture, bottomLeftTextureCoordinate).rgb;
     vec3 bottomRightColor = texture2D(inputImageTexture, bottomRightTextureCoordinate).rgb;
     vec4 centerColor = texture2D(inputImageTexture, textureCoordinate);
     vec3 leftColor = texture2D(inputImageTexture, leftTextureCoordinate).rgb;
     vec3 rightColor = texture2D(inputImageTexture, rightTextureCoordinate).rgb;
     vec3 topColor = texture2D(inputImageTexture, topTextureCoordinate).rgb;
     vec3 topRightColor = texture2D(inputImageTexture, topRightTextureCoordinate).rgb;
     vec3 topLeftColor = texture2D(inputImageTexture, topLeftTextureCoordinate).rgb;

     vec3 blurColor = topLeftColor * convolutionMatrix[0][0] + topColor * convolutionMatrix[0][1] + topRightColor * convolutionMatrix[0][2];
     blurColor += leftColor * convolutionMatrix[1][0] + centerColor.rgb * convolutionMatrix[1][1] + rightColor * convolutionMatrix[1][2];
     blurColor += bottomLeftColor * convolutionMatrix[2][0] + bottomColor * convolutionMatrix[2][1] + bottomRightColor * convolutionMatrix[2][2];

     float gaussianWeightTotal = convolutionMatrix[0][0] + convolutionMatrix[0][1] + convolutionMatrix[0][2] +
     convolutionMatrix[1][0] + convolutionMatrix[1][1] + convolutionMatrix[1][2] +
     convolutionMatrix[2][0] + convolutionMatrix[2][1] + convolutionMatrix[2][2];
     blurColor = blurColor / gaussianWeightTotal;

     vec3 sharpColor = 1.4 * centerColor.rgb - 0.4 * blurColor;

     return vec4(sharpColor, centerColor.a);
 }

 void main()
 {
     vec4 originColor = sharpenImage();
     vec3 texel = originColor.rgb;


     vec2 lookup;
     lookup.y = 0.5;

     lookup.x = texel.r;
     texel.r = texture2D(inputImageTexture2, lookup).r;

     lookup.x = texel.g;
     texel.g = texture2D(inputImageTexture2, lookup).g;

     lookup.x = texel.b;
     texel.b = texture2D(inputImageTexture2, lookup).b;

     float desaturatedColor;
     vec3 result;
     desaturatedColor = dot(desaturate, texel);


     lookup.x = desaturatedColor;
     result.r = texture2D(inputImageTexture3, lookup).r;
     lookup.x = desaturatedColor;
     result.g = texture2D(inputImageTexture3, lookup).g;
     lookup.x = desaturatedColor;
     result.b = texture2D(inputImageTexture3, lookup).b;

     texel = saturate * mix(texel, result, .5);

     vec2 tc = (2.0 * textureCoordinate) - 1.0;
     float d = dot(tc, tc);

     vec3 sampled;
     lookup.y = .5;

     /*
      lookup.x = texel.r;
      sampled.r = texture2D(inputImageTexture4, lookup).r;

      lookup.x = texel.g;
      sampled.g = texture2D(inputImageTexture4, lookup).g;

      lookup.x = texel.b;
      sampled.b = texture2D(inputImageTexture4, lookup).b;

      float value = smoothstep(0.0, 1.25, pow(d, 1.35)/1.65);
      texel = mix(texel, sampled, value);
      */

     //---

     lookup = vec2(d, texel.r);
     texel.r = texture2D(inputImageTexture4, lookup).r;
     lookup.y = texel.g;
     texel.g = texture2D(inputImageTexture4, lookup).g;
     lookup.y = texel.b;
     texel.b	= texture2D(inputImageTexture4, lookup).b;
     float value = smoothstep(0.0, 1.25, pow(d, 1.35)/1.65);

     //---

     lookup.x = texel.r;
     sampled.r = texture2D(inputImageTexture5, lookup).r;
     lookup.x = texel.g;
     sampled.g = texture2D(inputImageTexture5, lookup).g;
     lookup.x = texel.b;
     sampled.b = texture2D(inputImageTexture5, lookup).b;
     texel = mix(sampled, texel, value);


     lookup.x = texel.r;
     texel.r = texture2D(inputImageTexture6, lookup).r;
     lookup.x = texel.g;
     texel.g = texture2D(inputImageTexture6, lookup).g;
     lookup.x = texel.b;
     texel.b = texture2D(inputImageTexture6, lookup).b;

     gl_FragColor = vec4(texel, 1.0);
 }
