precision highp float;
 varying highp vec2 textureCoordinate;

 uniform sampler2D inputImageTexture;
 uniform sampler2D inputImageTexture2;
 uniform sampler2D inputImageTexture3;

 uniform float inputImageTextureHeight;
 uniform float inputImageTextureWidth;

 float texture2Size = 1024.0;

 uniform float p1;

 vec4 sharpenImage()
 {
     vec2 leftTextureCoordinate;
     vec2 rightTextureCoordinate;

     vec2 topTextureCoordinate;
     vec2 topLeftTextureCoordinate;
     vec2 topRightTextureCoordinate;

     vec2 bottomTextureCoordinate;
     vec2 bottomLeftTextureCoordinate;
     vec2 bottomRightTextureCoordinate;

     mat3 convolutionMatrix = mat3(0.0751, 0.1238, 0.0751, 0.1238, 0.2042, 0.1238, 0.0751, 0.1238, 0.0751);

     float texelWidth = 1.0 / inputImageTextureWidth;
     float texelHeight = 1.0 / inputImageTextureHeight;

     vec2 widthStep = vec2(texelWidth, 0.0);
     vec2 heightStep = vec2(0.0, texelHeight);
     vec2 widthHeightStep = vec2(texelWidth, texelHeight);
     vec2 widthNegativeHeightStep = vec2(texelWidth, -texelHeight);

     leftTextureCoordinate = textureCoordinate.xy - widthStep;
     rightTextureCoordinate = textureCoordinate.xy + widthStep;

     topTextureCoordinate = textureCoordinate.xy - heightStep;
     topLeftTextureCoordinate = textureCoordinate.xy - widthHeightStep;
     topRightTextureCoordinate = textureCoordinate.xy + widthNegativeHeightStep;

     bottomTextureCoordinate = textureCoordinate.xy + heightStep;
     bottomLeftTextureCoordinate = textureCoordinate.xy - widthNegativeHeightStep;
     bottomRightTextureCoordinate = textureCoordinate.xy + widthHeightStep;

     vec3 bottomColor = texture2D(inputImageTexture, bottomTextureCoordinate).rgb;
     vec3 bottomLeftColor = texture2D(inputImageTexture, bottomLeftTextureCoordinate).rgb;
     vec3 bottomRightColor = texture2D(inputImageTexture, bottomRightTextureCoordinate).rgb;
     vec4 centerColor = texture2D(inputImageTexture, textureCoordinate);
     vec3 leftColor = texture2D(inputImageTexture, leftTextureCoordinate).rgb;
     vec3 rightColor = texture2D(inputImageTexture, rightTextureCoordinate).rgb;
     vec3 topColor = texture2D(inputImageTexture, topTextureCoordinate).rgb;
     vec3 topRightColor = texture2D(inputImageTexture, topRightTextureCoordinate).rgb;
     vec3 topLeftColor = texture2D(inputImageTexture, topLeftTextureCoordinate).rgb;

     vec3 blurColor = topLeftColor * convolutionMatrix[0][0] + topColor * convolutionMatrix[0][1] + topRightColor * convolutionMatrix[0][2];
     blurColor += leftColor * convolutionMatrix[1][0] + centerColor.rgb * convolutionMatrix[1][1] + rightColor * convolutionMatrix[1][2];
     blurColor += bottomLeftColor * convolutionMatrix[2][0] + bottomColor * convolutionMatrix[2][1] + bottomRightColor * convolutionMatrix[2][2];

     float gaussianWeightTotal = convolutionMatrix[0][0] + convolutionMatrix[0][1] + convolutionMatrix[0][2] +
     convolutionMatrix[1][0] + convolutionMatrix[1][1] + convolutionMatrix[1][2] +
     convolutionMatrix[2][0] + convolutionMatrix[2][1] + convolutionMatrix[2][2];
     blurColor = blurColor / gaussianWeightTotal;

     vec3 sharpColor = 1.4 * centerColor.rgb - 0.4 * blurColor;

     return vec4(sharpColor, centerColor.a);
 }

 // // gray
 // float NCGray(vec4 color)
 //{
 //    float gray = 0.2125 * color.r + 0.7154 * color.g + 0.0721 * color.b;
 //
 //    return gray;
 //}
 //
 // // tone mapping
 // vec4 NCTonemapping(vec4 color)
 //{
 //
 //    vec4 mapped;
 //    mapped.r = texture2D(inputImageTexture2, vec2(color.r, 0.0)).r;
 //    mapped.g = texture2D(inputImageTexture2, vec2(color.g, 0.0)).g;
 //    mapped.b = texture2D(inputImageTexture2, vec2(color.b, 0.0)).b;
 //    mapped.a = color.a;
 //
 //    return mapped;
 //}
 //
 // // color control
 // vec4 NCColorControl(vec4 color, float saturation, float brightness, float contrast)
 //{
 //    float gray = NCGray(color);
 //
 //    color.rgb = vec3(saturation) * color.rgb + vec3(1.0-saturation) * vec3(gray);
 //    color.r = clamp(color.r, 0.0, 1.0);
 //    color.g = clamp(color.g, 0.0, 1.0);
 //    color.b = clamp(color.b, 0.0, 1.0);
 //
 //    color.rgb = vec3(contrast) * (color.rgb - vec3(0.5)) + vec3(0.5);
 //    color.r = clamp(color.r, 0.0, 1.0);
 //    color.g = clamp(color.g, 0.0, 1.0);
 //    color.b = clamp(color.b, 0.0, 1.0);
 //
 //    color.rgb = color.rgb + vec3(brightness);
 //    color.r = clamp(color.r, 0.0, 1.0);
 //    color.g = clamp(color.g, 0.0, 1.0);
 //    color.b = clamp(color.b, 0.0, 1.0);
 //
 //    return color;
 //}
 //
 // // hue adjust
 // vec4 NCHueAdjust(vec4 color, float hueAdjust)
 //{
 //    vec3 kRGBToYPrime = vec3(0.299, 0.587, 0.114);
 //    vec3 kRGBToI = vec3(0.595716, -0.274453, -0.321263);
 //    vec3 kRGBToQ = vec3(0.211456, -0.522591, 0.31135);
 //
 //    vec3 kYIQToR   = vec3(1.0, 0.9563, 0.6210);
 //    vec3 kYIQToG   = vec3(1.0, -0.2721, -0.6474);
 //    vec3 kYIQToB   = vec3(1.0, -1.1070, 1.7046);
 //
 //    float yPrime = dot(color.rgb, kRGBToYPrime);
 //    float I = dot(color.rgb, kRGBToI);
 //    float Q = dot(color.rgb, kRGBToQ);
 //
 //    float hue = atan(Q, I);
 //    float chroma  = sqrt (I * I + Q * Q);
 //
 //    hue -= hueAdjust;
 //
 //    Q = chroma * sin (hue);
 //    I = chroma * cos (hue);
 //
 //    color.r = dot(vec3(yPrime, I, Q), kYIQToR);
 //    color.g = dot(vec3(yPrime, I, Q), kYIQToG);
 //    color.b = dot(vec3(yPrime, I, Q), kYIQToB);
 //
 //    return color;
 //}
 //
 // // colorMatrix
 // vec4 NCColorMatrix(vec4 color, float red, float green, float blue, float alpha, vec4 bias)
 //{
 //    color = color * vec4(red, green, blue, alpha) + bias;
 //
 //    return color;
 //}
 //
 // multiply blend
 vec4 NCMultiplyBlend(vec4 overlay, vec4 base)
{
    vec4 outputColor;

    float a = overlay.a + base.a * (1.0 - overlay.a);

    //    // normal blend
    //    outputColor.r = (base.r * base.a + overlay.r * overlay.a * (1.0 - base.a))/a;
    //    outputColor.g = (base.g * base.a + overlay.g * overlay.a * (1.0 - base.a))/a;
    //    outputColor.b = (base.b * base.a + overlay.b * overlay.a * (1.0 - base.a))/a;


    // multiply blend
    outputColor.rgb = ((1.0-base.a) * overlay.rgb * overlay.a + (1.0-overlay.a) * base.rgb * base.a + overlay.a * base.a * overlay.rgb * base.rgb) / a;


    outputColor.a = a;

    return outputColor;
}
 //
 // // xy should be a integer position (e.g. pixel position on the screen)
 // // similar to a texture lookup but is only ALU
 // float PseudoRandom(vec2 co)
 //{
 //    //    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
 //    highp float a = 12.9898;
 //    highp float b = 78.233;
 //    highp float c = 43758.5453;
 //    highp float dt= dot(co.xy ,vec2(a,b));
 //    highp float sn= mod(dt,3.14);
 //    return fract(sin(sn) * c);
 //}
 //
 // void main()
 //{
 //    vec4 originColor = texture2D(inputImageTexture, textureCoordinate);
 //    vec4 color = texture2D(inputImageTexture, textureCoordinate);
 //
 //    color.a = 1.0;
 //
 //    // color control
 ////    color = NCColorControl(color, 0.6, -0.06, 0.75);
 //    color = NCColorControl(color, 0.5, 0.1, 0.9);
 //
 //    // rand
 //
 //    float x = textureCoordinate.x*inputImageTextureWidth/texture2Size;
 //    float y = textureCoordinate.y*inputImageTextureHeight/texture2Size;
 //
 //    vec4 rd = texture2D(inputImageTexture2, vec2(fract(x) , fract(y)));
 //
 ////    vec4 rd = texture2D(inputImageTexture2, vec2(textureCoordinate.x*inputImageTextureWidth/texture2Size - floor(inputImageTextureWidth/texture2Size), textureCoordinate.y*inputImageTextureHeight/texture2Size - floor(inputImageTextureHeight/texture2Size)));
 ////    vec4 rd = texture2D(inputImageTexture2, textureCoordinate);
 ////    float rand_number1 = PseudoRandom(textureCoordinate.xy);
 ////    float rand_number2 = PseudoRandom(textureCoordinate.yx);
 ////    float rand_number3 = PseudoRandom(vec2(rand_number1, rand_number2));
 ////    float rand_number4 = PseudoRandom(vec2(rand_number2, rand_number1));
 ////    float rand_number5 = PseudoRandom(vec2(rand_number3, rand_number4));
 //
 ////    vec4 rd = vec4(rand_number1, rand_number3, rand_number5, 1.0);
 //
 ////    if(rand_number4>0.2)
 ////        rd = vec4(1.0);
 //
 //    // rand color control
 ////    rd = NCColorControl(rd, 0.65, 0.1, 0.7);
 //    rd = NCColorControl(rd, 1.0, 0.4, 1.2);
 //
 //    // normal blend
 ////    rd.a *= 1.0;
 //    color = NCMultiplyBlend(rd, color);
 //
 //    // color matrix
 ////    color = NCColorMatrix(color, 1.0, 1.0, 1.0, 1.0, vec4(-0.1, -0.1, -0.1, 0));
 //    color = NCColorMatrix(color, 1.0, 1.0, 1.0, 1.0, vec4(-0.15, -0.15, -0.15, 0));
 //
 //    color.rgb = mix(originColor.rgb, color.rgb, p1);
 //    gl_FragColor = color;
 //}

 float Hue2RGB(float v1, float v2, float vH)
 {
     if (vH < 0.0) vH += 1.0;
     if (vH > 1.0) vH -= 1.0;
     if (6.0 * vH < 1.0) return v1 + (v2 - v1) * 6.0 * vH;
     if (2.0 * vH < 1.0) return v2;
     if (3.0 * vH < 2.0) return v1 + (v2 - v1) * ((2.0 / 3.0) - vH) * 6.0;

     return v1;
 }

 vec3 RGB2HSL(vec3 color)
 {
     float R;
     float G;
     float B;
     float maxV;
     float minV;
     float del_R;
     float del_G;
     float del_B;
     float del_Max;
     float H;
     float S;
     float L;

     R = color.r;
     G = color.g;
     B = color.b;

     minV = min(R, min(G, B));    //Min. value of RGB
     maxV = max(R, max(G, B));    //Max. value of RGB
     del_Max = maxV - minV;        //Delta RGB value

     L = (maxV + minV) / 2.0;

     if (del_Max == 0.0)           //This is a gray, no chroma...
     {
         H = 0.0;                  //HSL results = 0 ÷ 1
         S = 0.0;
     } else {                       //Chromatic data...
         if (L < 0.5) S = del_Max / (maxV + minV);
         else         S = del_Max / (2.0 - maxV - minV);

         del_R = (((maxV - R) / 6.0) + (del_Max / 2.0)) / del_Max;
         del_G = (((maxV - G) / 6.0) + (del_Max / 2.0)) / del_Max;
         del_B = (((maxV - B) / 6.0) + (del_Max / 2.0)) / del_Max;

         if      (R == maxV) H = del_B - del_G;
         else if (G == maxV) H = (1.0 / 3.0) + del_R - del_B;
         else if (B == maxV) H = (2.0 / 3.0) + del_G - del_R;

         if (H < 0.0)  H += 1.0;
         if (H > 1.0)  H -= 1.0;
     }

     return vec3(H, S, L);
 }

 vec3 HSL2RGB(vec3 hsl)
{
    float R;
    float G;
    float B;
    float var_1;
    float var_2;
    float H = hsl.x;
    float S = hsl.y;
    float L = hsl.z;

    if (S == 0.0)                       //HSL values = 0 ÷ 1
    {
        R = L;
        G = L;
        B = L;
    } else {
        if (L < 0.5) var_2 = L * (1.0 + S);
        else         var_2 = (L + S) - (S * L);

        var_1 = 2.0 * L - var_2;

        R = Hue2RGB(var_1, var_2, H + (1.0 / 3.0));
        G = Hue2RGB(var_1, var_2, H);
        B = Hue2RGB(var_1, var_2, H - (1.0 / 3.0));
    }
    return vec3(R,G,B);
}

 void main()
 {
     //     vec4 color = texture2D(inputImageTexture, textureCoordinate);
     vec4 color = sharpenImage();


     //curve
     vec4 mapped = color;
     mapped.r = texture2D(inputImageTexture2, vec2(color.r, 0.0)).r;
     mapped.g = texture2D(inputImageTexture2, vec2(color.g, 0.0)).g;
     mapped.b = texture2D(inputImageTexture2, vec2(color.b, 0.0)).b;

     //desaturation
     vec3 hsl = RGB2HSL(mapped.rgb);
     float s_off = -20.0/100.0;
     float s = hsl.y;
     float s_r = s*(s_off+1.0);
     hsl.y = clamp(s_r, 0.0, 1.0);
     mapped.rgb = HSL2RGB(hsl);

     // rand
     float x = textureCoordinate.x*inputImageTextureWidth/texture2Size;
     float y = textureCoordinate.y*inputImageTextureHeight/texture2Size;
     vec4 rd = texture2D(inputImageTexture3, vec2(fract(x), fract(y)));
     rd.rgb = clamp(vec3(1.4) - rd.rgb, 0.0, 1.0);

     //     mapped.rgb = mix(mapped.rgb, rd.rgb, 0.3*rd.a);
     rd.a = 0.3*rd.a;
     mapped = NCMultiplyBlend(rd, mapped);

     mapped.rgb = mix(color.rgb, mapped.rgb, p1);

     gl_FragColor = mapped;
 }