precision highp float;

 varying highp vec2 textureCoordinate;

 uniform sampler2D inputImageTexture;
 uniform sampler2D inputImageTexture2;  // brightness constrast curve 1
 uniform sampler2D inputImageTexture3;  // gradient palette
 uniform sampler2D inputImageTexture4;  // brightness constrast curve 2
 uniform sampler2D inputImageTexture5;  // scratch
 uniform sampler2D inputImageTexture6;  // noise
 uniform sampler2D inputImageTexture7;  // lomo

 uniform float p1;
 uniform float rand1;
 // uniform float rand2;

 vec3 black_white(vec3 color, float red, float yellow, float green, float cyan, float blue, float magenta)
 {
     float r = color.r;
     float g = color.g;
     float b = color.b;

     float max_c = max(max(r,g),b);
     float min_c = min(min(r,g),b);
     float mid_c = r + g + b - max_c - min_c;

     float ratio_max_mid;
     float ratio_max;

     if (min_c == r) {
         ratio_max_mid = cyan;
     } else if (min_c == g) {
         ratio_max_mid = magenta;
     } else if (min_c == b) {
         ratio_max_mid = yellow;
     }

     if (max_c == r) {
         ratio_max = red;
     } else if (max_c == g) {
         ratio_max = green;
     } else if (max_c == b) {
         ratio_max = blue;
     }

     return vec3( (max_c-mid_c)*ratio_max + (mid_c-min_c)*ratio_max_mid + min_c );
 }

 float rgb_to_l(vec3 color)
 {
     float minV;
     float maxV;
     if(color.r > color.g) {
         minV = min(color.g, color.b);
         maxV = max(color.r, color.b);
     } else {
         minV = min(color.r, color.b);
         maxV = max(color.g, color.b);
     }

     return clamp((minV+maxV)/2.0, 0.0, 1.0);
 }

 vec3 gradient_map(vec3 color)
 {
     vec3 RGBToYPrime = vec3(0.299, 0.587, 0.114);
     float lightness = dot(RGBToYPrime, color);

     vec3 curveColor;
     curveColor.r = texture2D(inputImageTexture3, vec2(lightness, 0.0)).r;
     curveColor.g = texture2D(inputImageTexture3, vec2(lightness, 0.0)).g;
     curveColor.b = texture2D(inputImageTexture3, vec2(lightness, 0.0)).b;

     return curveColor;
 }

 vec3 blend_overlay(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     float cb = base.r;
     float cs = src.r;

     vec3 blendColor;
     float blendValue;

     if (cb <= 0.5) {
         blendValue = 2.0*cb*cs;
     } else {
         blendValue = cs + (2.0*cb-1.0) - cs*(2.0*cb-1.0);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.r = blendValue;

     cb = base.g;
     cs = src.g;

     if (cb <= 0.5) {
         blendValue = 2.0*cb*cs;
     } else {
         blendValue = cs + (2.0*cb-1.0) - cs*(2.0*cb-1.0);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.g = blendValue;

     cb = base.b;
     cs = src.b;

     if (cb <= 0.5) {
         blendValue = 2.0*cb*cs;
     } else {
         blendValue = cs + (2.0*cb-1.0) - cs*(2.0*cb-1.0);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);
     blendColor.b = blendValue;

     return blendColor;
 }

 vec3 blend_screen(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     float cb = base.r;
     float cs = src.r;

     vec3 blendColor;
     float blendValue;

     blendValue = cb + cs*(1.0-cb);

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);
     blendColor.r = blendValue;

     cb = base.g;
     cs = src.g;

     blendValue = cb + cs*(1.0-cb);

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);
     blendColor.g = blendValue;

     cb = base.b;
     cs = src.b;

     blendValue = cb + cs*(1.0-cb);

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);
     blendColor.b = blendValue;

     return blendColor;
 }

 float softlight_d( float x )
 {
     float v;

     if (x<=0.25) {
         v = ((16.0*x-12.0)*x+4.0)*x;
     } else {
         v = sqrt(x);
     }

     return v;
 }

 vec3 blend_softlight(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     float cb = base.r;
     float cs = src.r;

     vec3 blendColor;
     float blendValue;

     if (cs <= 0.5) {
         blendValue = cb - (1.0-2.0*cs)*cb*(1.0-cb);
     } else {
         blendValue = cb + (2.0*cs-1.0)*(softlight_d(cb)-cb);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.r = blendValue;

     cb = base.g;
     cs = src.g;

     if (cs <= 0.5) {
         blendValue = cb - (1.0-2.0*cs)*cb*(1.0-cb);
     } else {
         blendValue = cb + (2.0*cs-1.0)*(softlight_d(cb)-cb);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.g = blendValue;

     cb = base.b;
     cs = src.b;

     if (cs <= 0.5) {
         blendValue = cb - (1.0-2.0*cs)*cb*(1.0-cb);
     } else {
         blendValue = cb + (2.0*cs-1.0)*(softlight_d(cb)-cb);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.b = blendValue;

     return blendColor;
 }

 vec3 blend_normal(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     vec3 blendValue;

     blendValue = src;

     blendValue = (1.0-alpha_src)*base + alpha_src*((1.0-alpha_base)*src + alpha_base*blendValue);

     return blendValue;
 }


 void main()
 {
     vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);

     // bright contrast 1
     vec3 brightnessContrastColor1;
     brightnessContrastColor1.r = texture2D(inputImageTexture2, vec2(textureColor.r, 0.0)).r;
     brightnessContrastColor1.g = texture2D(inputImageTexture2, vec2(textureColor.g, 0.0)).g;
     brightnessContrastColor1.b = texture2D(inputImageTexture2, vec2(textureColor.b, 0.0)).b;

     // gradient 1
     vec3 gradientColor;
     gradientColor = gradient_map(brightnessContrastColor1);

     // overlay blend
     vec3 overlayColor;
     overlayColor = blend_overlay(brightnessContrastColor1, gradientColor, 1.0, 1.0);

     // black white
     vec3 grayColor;
     grayColor = black_white(overlayColor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

     // bright contrast 2
     vec3 brightnessContrastColor2;
     brightnessContrastColor2.r = texture2D(inputImageTexture4, vec2(grayColor.r, 0.0)).r;
     brightnessContrastColor2.g = texture2D(inputImageTexture4, vec2(grayColor.g, 0.0)).g;
     brightnessContrastColor2.b = texture2D(inputImageTexture4, vec2(grayColor.b, 0.0)).b;

     // scratch 1
     vec4 scratchColor1 = texture2D(inputImageTexture5, textureCoordinate);
     vec3 screenColor1 = blend_screen(brightnessContrastColor2, scratchColor1.rgb, 1.0, 1.0);

     // noise
     vec2 textureCoordinateRand1 = textureCoordinate+vec2(rand1, rand1);
     if (textureCoordinateRand1.x > 1.0)
         textureCoordinateRand1.x = textureCoordinateRand1.x - 1.0;
     if (textureCoordinateRand1.y > 1.0)
         textureCoordinateRand1.y = textureCoordinateRand1.y - 1.0;

     vec4 noise1 = texture2D(inputImageTexture6, textureCoordinateRand1);
     vec3 noiseColor = blend_softlight(screenColor1, noise1.rgb, 1.0, 1.0);

     // lomo
     vec4 lomo1 = texture2D(inputImageTexture7, textureCoordinate);
     vec3 lomoColor = blend_softlight(noiseColor, lomo1.rgb, 1.0, 1.0);
     //
     //     //     // scratch 2
     //     //     vec2 textureCoordinateRand2 = textureCoordinate+vec2(rand2, rand2);
     //     //     if (textureCoordinateRand2.x > 1.0)
     //     //         textureCoordinateRand2.x = textureCoordinateRand2.x - 1.0;
     //     //     if (textureCoordinateRand2.y > 1.0)
     //     //         textureCoordinateRand2.y = textureCoordinateRand2.y - 1.0;
     //     //
     //     //     vec4 scratchColor2 = texture2D(inputImageTexture6, textureCoordinateRand2);
     //     //     vec3 screenColor2 = blend_screen(screenColor1, scratchColor2.rgb, 1.0, 0.8);
     //
     gl_FragColor = vec4(mix(textureColor.rgb, lomoColor.rgb, p1), textureColor.a);
 }