precision highp float;

 varying highp vec2 textureCoordinate;

 uniform sampler2D inputImageTexture;
 uniform sampler2D inputImageTexture2;

 uniform float inputImageTextureHeight;
 uniform float inputImageTextureWidth;

 uniform float p1;

 vec4 sharpenImage()
 {
     vec2 leftTextureCoordinate;
     vec2 rightTextureCoordinate;

     vec2 topTextureCoordinate;
     vec2 topLeftTextureCoordinate;
     vec2 topRightTextureCoordinate;

     vec2 bottomTextureCoordinate;
     vec2 bottomLeftTextureCoordinate;
     vec2 bottomRightTextureCoordinate;

     mat3 convolutionMatrix = mat3(0.0751, 0.1238, 0.0751, 0.1238, 0.2042, 0.1238, 0.0751, 0.1238, 0.0751);

     float texelWidth = 1.0 / inputImageTextureWidth;
     float texelHeight = 1.0 / inputImageTextureHeight;

     vec2 widthStep = vec2(texelWidth, 0.0);
     vec2 heightStep = vec2(0.0, texelHeight);
     vec2 widthHeightStep = vec2(texelWidth, texelHeight);
     vec2 widthNegativeHeightStep = vec2(texelWidth, -texelHeight);

     leftTextureCoordinate = textureCoordinate.xy - widthStep;
     rightTextureCoordinate = textureCoordinate.xy + widthStep;

     topTextureCoordinate = textureCoordinate.xy - heightStep;
     topLeftTextureCoordinate = textureCoordinate.xy - widthHeightStep;
     topRightTextureCoordinate = textureCoordinate.xy + widthNegativeHeightStep;

     bottomTextureCoordinate = textureCoordinate.xy + heightStep;
     bottomLeftTextureCoordinate = textureCoordinate.xy - widthNegativeHeightStep;
     bottomRightTextureCoordinate = textureCoordinate.xy + widthHeightStep;

     vec3 bottomColor = texture2D(inputImageTexture, bottomTextureCoordinate).rgb;
     vec3 bottomLeftColor = texture2D(inputImageTexture, bottomLeftTextureCoordinate).rgb;
     vec3 bottomRightColor = texture2D(inputImageTexture, bottomRightTextureCoordinate).rgb;
     vec4 centerColor = texture2D(inputImageTexture, textureCoordinate);
     vec3 leftColor = texture2D(inputImageTexture, leftTextureCoordinate).rgb;
     vec3 rightColor = texture2D(inputImageTexture, rightTextureCoordinate).rgb;
     vec3 topColor = texture2D(inputImageTexture, topTextureCoordinate).rgb;
     vec3 topRightColor = texture2D(inputImageTexture, topRightTextureCoordinate).rgb;
     vec3 topLeftColor = texture2D(inputImageTexture, topLeftTextureCoordinate).rgb;

     vec3 blurColor = topLeftColor * convolutionMatrix[0][0] + topColor * convolutionMatrix[0][1] + topRightColor * convolutionMatrix[0][2];
     blurColor += leftColor * convolutionMatrix[1][0] + centerColor.rgb * convolutionMatrix[1][1] + rightColor * convolutionMatrix[1][2];
     blurColor += bottomLeftColor * convolutionMatrix[2][0] + bottomColor * convolutionMatrix[2][1] + bottomRightColor * convolutionMatrix[2][2];

     float gaussianWeightTotal = convolutionMatrix[0][0] + convolutionMatrix[0][1] + convolutionMatrix[0][2] +
     convolutionMatrix[1][0] + convolutionMatrix[1][1] + convolutionMatrix[1][2] +
     convolutionMatrix[2][0] + convolutionMatrix[2][1] + convolutionMatrix[2][2];
     blurColor = blurColor / gaussianWeightTotal;

     vec3 sharpColor = 1.4 * centerColor.rgb - 0.4 * blurColor;

     return vec4(sharpColor, centerColor.a);
 }

 float RGB2L(vec3 color)
 {
     float minV;
     float maxV;
     if(color.r > color.g) {
         minV = min(color.g, color.b);
         maxV = max(color.r, color.b);
     } else {
         minV = min(color.r, color.b);
         maxV = max(color.g, color.b);
     }

     return clamp((minV+maxV)/2.0, 0.0, 1.0);
 }

 vec3 RGB2HSL(vec3 color)
 {
     float R;
     float G;
     float B;
     float maxV;
     float minV;
     float del_R;
     float del_G;
     float del_B;
     float del_Max;
     float H;
     float S;
     float L;

     R = color.r;
     G = color.g;
     B = color.b;

     minV = min(R, min(G, B));    //Min. value of RGB
     maxV = max(R, max(G, B));    //Max. value of RGB
     del_Max = maxV - minV;        //Delta RGB value

     L = (maxV + minV) / 2.0;

     if (del_Max == 0.0)           //This is a gray, no chroma...
     {
         H = 0.0;                  //HSL results = 0 ÷ 1
         S = 0.0;
     } else {                       //Chromatic data...
         if (L < 0.5) S = del_Max / (maxV + minV);
         else         S = del_Max / (2.0 - maxV - minV);

         del_R = (((maxV - R) / 6.0) + (del_Max / 2.0)) / del_Max;
         del_G = (((maxV - G) / 6.0) + (del_Max / 2.0)) / del_Max;
         del_B = (((maxV - B) / 6.0) + (del_Max / 2.0)) / del_Max;

         if      (R == maxV) H = del_B - del_G;
         else if (G == maxV) H = (1.0 / 3.0) + del_R - del_B;
         else if (B == maxV) H = (2.0 / 3.0) + del_G - del_R;

         if (H < 0.0)  H += 1.0;
         if (H > 1.0)  H -= 1.0;
     }

     return vec3(H, S, L);
 }

 float Hue2RGB(float v1, float v2, float vH)
 {
     if (vH < 0.0) vH += 1.0;
     if (vH > 1.0) vH -= 1.0;
     if (6.0 * vH < 1.0) return v1 + (v2 - v1) * 6.0 * vH;
     if (2.0 * vH < 1.0) return v2;
     if (3.0 * vH < 2.0) return v1 + (v2 - v1) * ((2.0 / 3.0) - vH) * 6.0;

     return v1;
 }

 vec3 HSL2RGB(vec3 hsl)
 {
     float R;
     float G;
     float B;
     float var_1;
     float var_2;
     float H = hsl.x;
     float S = hsl.y;
     float L = hsl.z;

     if (S == 0.0)                       //HSL values = 0 ÷ 1
     {
         R = L;
         G = L;
         B = L;
     } else {
         if (L < 0.5) var_2 = L * (1.0 + S);
         else         var_2 = (L + S) - (S * L);

         var_1 = 2.0 * L - var_2;

         R = Hue2RGB(var_1, var_2, H + (1.0 / 3.0));
         G = Hue2RGB(var_1, var_2, H);
         B = Hue2RGB(var_1, var_2, H - (1.0 / 3.0));
     }
     return vec3(R,G,B);
 }

 vec3 hueSaturation(vec3 color, float hue_offset, float lightness_offset, float saturation_offset)
 {
     vec3 hsl = RGB2HSL(color);
     float h = hsl.x;
     float s = hsl.y;
     float l = hsl.z;

     float hue_offset_n = hue_offset;
     float lightness_offset_n = lightness_offset * 0.5;
     float saturation_offset_n = saturation_offset;

     float hue_n;
     float lightness_n;
     float saturation_n;

     float h_n = (h + hue_offset_n);
     if (h_n < 0.0)
         hue_n = 1.0 + h_n;
     else if (h_n > 1.0)
         hue_n = h_n - 1.0;
     else
         hue_n = h_n;

     lightness_offset_n = clamp(lightness_offset_n, -1.0, 1.0);
     if (lightness_offset_n < 0.0)
         lightness_n = l * (1.0 + lightness_offset_n);
     else
         lightness_n = l + ((1.0 - l) * lightness_offset_n);

     saturation_offset_n = clamp(saturation_offset_n, -1.0, 1.0);
     saturation_n = clamp(s * (1.0 + saturation_offset_n), 0.0, 1.0);

     return HSL2RGB(vec3(hue_n, saturation_n, lightness_n));
 }

 float colorBalanceMap(float value, float lightness, float shadows, float midtones, float highlights)
 {
     float a = 0.251;
     float b = 0.3333;
     float scale = 0.7;

     shadows *= clamp((lightness-b)/-a + 0.5, 0.0, 1.0) * scale;
     midtones *= clamp((lightness - b) / a + 0.5, 0.0, 1.0) * clamp((lightness + b - 1.0) / -a + 0.5, 0.0, 1.0) * scale;
     highlights *= clamp((lightness + b - 1.0) / a + 0.5, 0.0, 1.0) * scale;
     value += shadows;
     value += midtones;
     value += highlights;

     return clamp(value, 0.0, 1.0);
 }

 void main()
 {
     // sharp
     vec4 textureColor = sharpenImage();

     // curve tone
     vec4 curveColor = textureColor;
     curveColor.r = texture2D(inputImageTexture2, vec2(textureColor.r, 0.0)).r;
     curveColor.g = texture2D(inputImageTexture2, vec2(textureColor.g, 0.0)).g;
     curveColor.b = texture2D(inputImageTexture2, vec2(textureColor.b, 0.0)).b;

     // hue/saturation
     vec4 hslColor = curveColor;
     hslColor.rgb = hueSaturation(curveColor.rgb, -2.0/360.0, 0.0, 0.43);

     // color balance
     vec4 balanceColor = hslColor;
     float lightness = RGB2L(hslColor.rgb);

     balanceColor.r = colorBalanceMap(hslColor.r, lightness, 0.05,  0.05,  0.0);
     balanceColor.g = colorBalanceMap(hslColor.g, lightness, 0.0,   0.0,   0.0);
     balanceColor.b = colorBalanceMap(hslColor.b, lightness, 0.0,  -0.03, -0.02);

     balanceColor.rgb = clamp(balanceColor.rgb, 0.0, 1.0);
     vec3 originalHSLColor = RGB2HSL(hslColor.rgb);
     vec3 nowHSLColor = RGB2HSL(balanceColor.rgb);
     nowHSLColor.z = originalHSLColor.z;
     balanceColor.rgb = HSL2RGB(nowHSLColor.rgb);

     vec4 result = textureColor;
     result.rgb = mix(textureColor.rgb, balanceColor.rgb, p1);

     gl_FragColor = result;
 }