 precision highp float;

 varying vec2 textureCoordinate;

 uniform sampler2D inputImageTexture;
 uniform sampler2D inputImageTexture2;
 uniform sampler2D inputImageTexture3;
 uniform sampler2D inputImageTexture4;

 uniform float p1;

 vec3 blend_normal(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     vec3 blendValue;

     blendValue = src;

     blendValue = (1.0-alpha_src)*base + alpha_src*((1.0-alpha_base)*src + alpha_base*blendValue);

     return blendValue;
 }

 vec3 blend_multiply(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     vec3 blendValue;

     blendValue = base * src;

     blendValue = (1.0-alpha_src)*base + alpha_src*((1.0-alpha_base)*src + alpha_base*blendValue);

     return blendValue;
 }

 vec3 blend_screen(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     vec3 blendValue;

     blendValue = base + src - (base * src);

     blendValue = (1.0-alpha_src)*base + alpha_src*((1.0-alpha_base)*src + alpha_base*blendValue);

     return blendValue;
 }

 float softlight_d( float x )
 {
     float v;

     if (x<=0.25) {
         v = ((16.0*x-12.0)*x+4.0)*x;
     } else {
         v = sqrt(x);
     }

     return v;
 }

 vec3 blend_softlight(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     float cb = base.r;
     float cs = src.r;

     vec3 blendColor;
     float blendValue;

     if (cs <= 0.5) {
         blendValue = cb - (1.0-2.0*cs)*cb*(1.0-cb);
     } else {
         blendValue = cb + (2.0*cs-1.0)*(softlight_d(cb)-cb);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.r = blendValue;

     cb = base.g;
     cs = src.g;

     if (cs <= 0.5) {
         blendValue = cb - (1.0-2.0*cs)*cb*(1.0-cb);
     } else {
         blendValue = cb + (2.0*cs-1.0)*(softlight_d(cb)-cb);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.g = blendValue;

     cb = base.b;
     cs = src.b;

     if (cs <= 0.5) {
         blendValue = cb - (1.0-2.0*cs)*cb*(1.0-cb);
     } else {
         blendValue = cb + (2.0*cs-1.0)*(softlight_d(cb)-cb);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.b = blendValue;

     return blendColor;
 }

 float lum(vec3 c)
 {
     return dot(c, vec3(0.3, 0.59, 0.11));
 }

 vec3 clipcolor(vec3 c)
 {
     float l = lum(c);
     float n = min(min(c.r, c.g), c.b);
     float x = max(max(c.r, c.g), c.b);

     if (n < 0.0) {
         c.r = l + ((c.r - l) * l) / (l - n);
         c.g = l + ((c.g - l) * l) / (l - n);
         c.b = l + ((c.b - l) * l) / (l - n);
     }
     if (x > 1.0) {
         c.r = l + ((c.r - l) * (1.0 - l)) / (x - l);
         c.g = l + ((c.g - l) * (1.0 - l)) / (x - l);
         c.b = l + ((c.b - l) * (1.0 - l)) / (x - l);
     }

     return c;
 }

 vec3 setlum(vec3 c, float l) {
     float d = l - lum(c);
     c = c + vec3(d);
     return clipcolor(c);
 }

 vec3 blend_color(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     vec3 blendValue;

     blendValue = setlum(src, lum(base));

     blendValue = (1.0-alpha_src)*base + alpha_src*((1.0-alpha_base)*src + alpha_base*blendValue);

     return blendValue;
 }

 vec3 levels_control(vec3 color, vec3 minInput, vec3 gamma, vec3 maxInput, vec3 minOutput, vec3 maxOutput)
 {
     vec3 levelInputRange = min(max(color - minInput, vec3(0.0)) / (maxInput - minInput), vec3(1.0));

     vec3 levelInput = pow(levelInputRange, 1.0/gamma);

     vec3 levelOuput = mix(minOutput, maxOutput, levelInput);

     return levelOuput;
 }

 void main()
 {
     mediump vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);

     // curve
     vec3 curveColor;
     curveColor.r = texture2D(inputImageTexture2, vec2(textureColor.r, 0.0)).r;
     curveColor.g = texture2D(inputImageTexture2, vec2(textureColor.g, 0.0)).g;
     curveColor.b = texture2D(inputImageTexture2, vec2(textureColor.b, 0.0)).b;

     // level
     vec3 levelMinimum = vec3(0.0, 0.0, 0.0);
     vec3 levelMiddle = vec3(1.0, 1.0, 1.0);
     vec3 levelMaximum = vec3(1.0, 1.0, 1.0);
     vec3 minOutput = vec3(20.0/255.0, 0.0, 43.0/255.0);
     vec3 maxOutput = vec3(1.0, 1.0, 1.0);
     vec3 levelColor = levels_control(curveColor, levelMinimum, levelMiddle, levelMaximum, minOutput, maxOutput);

     // color 1
     vec3 blendColor1;
     blendColor1 = blend_color(levelColor, vec3(220.0/255.0, 110.0/255.0, 149.0/255.0), 1.0, 0.1);

     // contrast
     vec3 contrastColor;
     contrastColor.r = texture2D(inputImageTexture3, vec2(blendColor1.r, 0.0)).r;
     contrastColor.g = texture2D(inputImageTexture3, vec2(blendColor1.g, 0.0)).g;
     contrastColor.b = texture2D(inputImageTexture3, vec2(blendColor1.b, 0.0)).b;

     // color 2
     vec3 blendColor2;
     blendColor2 = blend_softlight(contrastColor, vec3(68.0/255.0, 147.0/255.0, 176.0/255.0), 1.0, 0.8);

     // color 3
     vec3 blendColor3;
     blendColor3 = blend_softlight(blendColor2, vec3(220.0/255.0, 172.0/255.0, 110.0/255.0), 1.0, 1.0);

     // color 4
     vec3 blendColor4;
     blendColor4 = blend_softlight(blendColor3, vec3(185.0/255.0, 100.0/255.0, 234.0/255.0), 1.0, 0.1);

     // gradient map
     vec3 blendColor5;
     vec4 screenColor = texture2D(inputImageTexture4, textureCoordinate);
     blendColor5 = blend_screen(blendColor4, screenColor.rgb, 1.0, screenColor.a * 0.7);

     // color 5
     vec3 blendColor6;
     blendColor6 = blend_multiply(blendColor5, vec3(229.0/255.0, 196.0/255.0, 144.0/255.0), 1.0, 0.4);

     gl_FragColor = vec4(mix(textureColor.rgb, blendColor6.rgb, p1),textureColor.a);
 }