precision mediump float;

 varying vec2 textureCoordinate;

 uniform sampler2D inputImageTexture;
 uniform sampler2D inputImageTexture2;
 uniform sampler2D inputImageTexture3;
 uniform sampler2D inputImageTexture4;
 uniform sampler2D inputImageTexture5;

 uniform float p1;
 uniform float p2;

 float Hue2RGB(float v1, float v2, float vH)
 {
     if (vH < 0.0) vH += 1.0;
     if (vH > 1.0) vH -= 1.0;
     if (6.0 * vH < 1.0) return v1 + (v2 - v1) * 6.0 * vH;
     if (2.0 * vH < 1.0) return v2;
     if (3.0 * vH < 2.0) return v1 + (v2 - v1) * ((2.0 / 3.0) - vH) * 6.0;

     return v1;
 }

 vec3 RGB2HSL(vec3 color)
 {
     float R;
     float G;
     float B;
     float maxV;
     float minV;
     float del_R;
     float del_G;
     float del_B;
     float del_Max;
     float H;
     float S;
     float L;

     R = color.r;
     G = color.g;
     B = color.b;

     minV = min(R, min(G, B));    //Min. value of RGB
     maxV = max(R, max(G, B));    //Max. value of RGB
     del_Max = maxV - minV;        //Delta RGB value

     L = (maxV + minV) / 2.0;

     if (del_Max == 0.0)           //This is a gray, no chroma...
     {
         H = 0.0;                  //HSL results = 0 ÷ 1
         S = 0.0;
     } else {                       //Chromatic data...
         if (L < 0.5) S = del_Max / (maxV + minV);
         else         S = del_Max / (2.0 - maxV - minV);

         del_R = (((maxV - R) / 6.0) + (del_Max / 2.0)) / del_Max;
         del_G = (((maxV - G) / 6.0) + (del_Max / 2.0)) / del_Max;
         del_B = (((maxV - B) / 6.0) + (del_Max / 2.0)) / del_Max;

         if      (R == maxV) H = del_B - del_G;
         else if (G == maxV) H = (1.0 / 3.0) + del_R - del_B;
         else if (B == maxV) H = (2.0 / 3.0) + del_G - del_R;

         if (H < 0.0)  H += 1.0;
         if (H > 1.0)  H -= 1.0;
     }

     return vec3(H, S, L);
 }

 vec3 HSL2RGB(vec3 hsl)
 {
    float R;
    float G;
    float B;
    float var_1;
    float var_2;
    float H = hsl.x;
    float S = hsl.y;
    float L = hsl.z;

    if (S == 0.0)                       //HSL values = 0 ÷ 1
    {
        R = L;
        G = L;
        B = L;
    } else {
        if (L < 0.5) var_2 = L * (1.0 + S);
        else         var_2 = (L + S) - (S * L);

        var_1 = 2.0 * L - var_2;

        R = Hue2RGB(var_1, var_2, H + (1.0 / 3.0));
        G = Hue2RGB(var_1, var_2, H);
        B = Hue2RGB(var_1, var_2, H - (1.0 / 3.0));
    }
    return vec3(R,G,B);
 }

 void main()
 {
     vec4 color = texture2D(inputImageTexture, textureCoordinate);
     vec4 blurColor = texture2D(inputImageTexture2, textureCoordinate);
     vec4 mapped = blurColor;
     mapped.rgb = vec3(1.0);

     float whiteStrength = p2;

     if(whiteStrength == 3.0) {
              mapped.r = texture2D(inputImageTexture3, vec2(blurColor.r, 0.0)).r;
              mapped.g = texture2D(inputImageTexture3, vec2(blurColor.g, 0.0)).g;
              mapped.b = texture2D(inputImageTexture3, vec2(blurColor.b, 0.0)).b;
          } else if (whiteStrength == 2.0) {
              mapped.r = texture2D(inputImageTexture4, vec2(blurColor.r, 0.0)).r;
              mapped.g = texture2D(inputImageTexture4, vec2(blurColor.g, 0.0)).g;
              mapped.b = texture2D(inputImageTexture4, vec2(blurColor.b, 0.0)).b;
          } else {
              mapped.r = texture2D(inputImageTexture5, vec2(blurColor.r, 0.0)).r;
              mapped.g = texture2D(inputImageTexture5, vec2(blurColor.g, 0.0)).g;
              mapped.b = texture2D(inputImageTexture5, vec2(blurColor.b, 0.0)).b;
          }

     vec3 hsv = RGB2HSL(mapped.rgb);
     hsv.x =  hsv.x - 2.0/180.0;
     if (hsv.x < 0.0)
         hsv.x = 1.0 + hsv.x;
     hsv.y = hsv.y - 2.0/100.0;
     clamp(hsv.y, 0.0, 1.0);
     hsv.z = hsv.z + 2.0/100.0;
     clamp(hsv.z, 0.0, 1.0);
     mapped.rgb = HSL2RGB(hsv);

     gl_FragColor = vec4(mix(color.rgb, mapped.rgb, p1), color.w);
 }