precision highp float;

 varying highp vec2 textureCoordinate;

 uniform sampler2D inputImageTexture;
 uniform sampler2D inputImageTexture2;  // brightness constrast curve
 uniform sampler2D inputImageTexture3;  // intensity curve
 uniform sampler2D inputImageTexture4;  // gradient palette 4
 uniform sampler2D inputImageTexture5;  // gradient palette 5
 uniform sampler2D inputImageTexture6;  // gradient palette 6

 uniform mediump float p1;

 vec3 black_white(vec3 color, float red, float yellow, float green, float cyan, float blue, float magenta)
 {
     float r = color.r;
     float g = color.g;
     float b = color.b;

     float max_c = max(max(r,g),b);
     float min_c = min(min(r,g),b);
     float mid_c = r + g + b - max_c - min_c;

     float ratio_max_mid;
     float ratio_max;

     if (min_c == r) {
         ratio_max_mid = cyan;
     } else if (min_c == g) {
         ratio_max_mid = magenta;
     } else if (min_c == b) {
         ratio_max_mid = yellow;
     }

     if (max_c == r) {
         ratio_max = red;
     } else if (max_c == g) {
         ratio_max = green;
     } else if (max_c == b) {
         ratio_max = blue;
     }

     return vec3( (max_c-mid_c)*ratio_max + (mid_c-min_c)*ratio_max_mid + min_c );
 }

 float rgb_to_l(vec3 color)
 {
     float minV;
     float maxV;
     if(color.r > color.g) {
         minV = min(color.g, color.b);
         maxV = max(color.r, color.b);
     } else {
         minV = min(color.r, color.b);
         maxV = max(color.g, color.b);
     }

     return clamp((minV+maxV)/2.0, 0.0, 1.0);
 }

 vec3 gradient_map(vec3 color, int index)
 {
     vec3 RGBToYPrime = vec3(0.299, 0.587, 0.114);
     float lightness = dot(RGBToYPrime, color);

     vec3 curveColor;
     if (index == 4) {
         curveColor.r = texture2D(inputImageTexture4, vec2(color.r, 0.0)).r;
         curveColor.g = texture2D(inputImageTexture4, vec2(color.g, 0.0)).g;
         curveColor.b = texture2D(inputImageTexture4, vec2(color.b, 0.0)).b;
     } else if(index == 5) {
         curveColor.r = texture2D(inputImageTexture5, vec2(color.r, 0.0)).r;
         curveColor.g = texture2D(inputImageTexture5, vec2(color.g, 0.0)).g;
         curveColor.b = texture2D(inputImageTexture5, vec2(color.b, 0.0)).b;
     } else if(index == 6) {
         curveColor.r = texture2D(inputImageTexture6, vec2(color.r, 0.0)).r;
         curveColor.g = texture2D(inputImageTexture6, vec2(color.g, 0.0)).g;
         curveColor.b = texture2D(inputImageTexture6, vec2(color.b, 0.0)).b;
     }

     return curveColor;
 }

 float softlight_d( float x )
 {
     float v;

     if (x<=0.25) {
         v = ((16.0*x-12.0)*x+4.0)*x;
     } else {
         v = sqrt(x);
     }

     return v;
 }

 vec3 blend_softlight(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     float cb = base.r;
     float cs = src.r;

     vec3 blendColor;
     float blendValue;

     if (cs <= 0.5) {
         blendValue = cb - (1.0-2.0*cs)*cb*(1.0-cb);
     } else {
         blendValue = cb + (2.0*cs-1.0)*(softlight_d(cb)-cb);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.r = blendValue;

     cb = base.g;
     cs = src.g;

     if (cs <= 0.5) {
         blendValue = cb - (1.0-2.0*cs)*cb*(1.0-cb);
     } else {
         blendValue = cb + (2.0*cs-1.0)*(softlight_d(cb)-cb);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.g = blendValue;

     cb = base.b;
     cs = src.b;

     if (cs <= 0.5) {
         blendValue = cb - (1.0-2.0*cs)*cb*(1.0-cb);
     } else {
         blendValue = cb + (2.0*cs-1.0)*(softlight_d(cb)-cb);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.b = blendValue;

     return blendColor;
 }

 vec3 blend_normal(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     vec3 blendValue;

     blendValue = src;

     blendValue = (1.0-alpha_src)*base + alpha_src*((1.0-alpha_base)*src + alpha_base*blendValue);

     return blendValue;
 }

 void main()
 {
     vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);

     // bright contrast
     vec3 brightnessContrastColor;
     brightnessContrastColor.r = texture2D(inputImageTexture2, vec2(textureColor.r, 0.0)).r;
     brightnessContrastColor.g = texture2D(inputImageTexture2, vec2(textureColor.g, 0.0)).g;
     brightnessContrastColor.b = texture2D(inputImageTexture2, vec2(textureColor.b, 0.0)).b;

     // intensity
     vec3 instensityColor;
     instensityColor.r = texture2D(inputImageTexture3, vec2(brightnessContrastColor.r, 0.0)).r;
     instensityColor.g = texture2D(inputImageTexture3, vec2(brightnessContrastColor.g, 0.0)).g;
     instensityColor.b = texture2D(inputImageTexture3, vec2(brightnessContrastColor.b, 0.0)).b;

     // black white
     vec3 grayColor;
     grayColor = black_white(instensityColor, 0.4, 0.6, 0.4, 0.6, -0.56, 0.8);

     // gradient 4
     vec3 gradientColor4;
     gradientColor4 = gradient_map(grayColor, 4);

     // soft light blend
     vec3 softlightColor4;
     softlightColor4 = blend_softlight(grayColor, gradientColor4, 1.0, 0.85);

     // gradient 5
     vec3 gradientColor5;
     gradientColor5 = gradient_map(softlightColor4, 5);

     // soft light blend
     vec3 softlightColor5;
     softlightColor5 = blend_softlight(softlightColor4, gradientColor5, 1.0, 0.32);

     // gradient 6
     vec3 gradientColor6;
     gradientColor6 = gradient_map(softlightColor5, 6);

     // normal blend
     vec3 normalColor;
     normalColor = blend_normal(softlightColor5, gradientColor6, 1.0, 0.49);

     gl_FragColor = vec4(mix(textureColor.rgb, normalColor, p1), textureColor.a);
 }