precision mediump float;
 varying mediump vec2 textureCoordinate;

 uniform sampler2D inputImageTexture;
 uniform sampler2D inputImageTexture2;
 uniform sampler2D inputImageTexture3;

 uniform float p1;

 vec3 blend_screen(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     float cb = base.r;
     float cs = src.r;

     vec3 blendColor;
     float blendValue;

     blendValue = cb + cs*(1.0-cb);

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);
     blendColor.r = blendValue;

     cb = base.g;
     cs = src.g;

     blendValue = cb + cs*(1.0-cb);

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);
     blendColor.g = blendValue;

     cb = base.b;
     cs = src.b;

     blendValue = cb + cs*(1.0-cb);

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);
     blendColor.b = blendValue;

     return blendColor;
 }

 float softlight_d( float x )
 {
     float v;

     if (x<=0.25) {
         v = ((16.0*x-12.0)*x+4.0)*x;
     } else {
         v = sqrt(x);
     }

     return v;
 }

 vec3 blend_softlight(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     float cb = base.r;
     float cs = src.r;

     vec3 blendColor;
     float blendValue;

     if (cs <= 0.5) {
         blendValue = cb - (1.0-2.0*cs)*cb*(1.0-cb);
     } else {
         blendValue = cb + (2.0*cs-1.0)*(softlight_d(cb)-cb);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.r = blendValue;

     cb = base.g;
     cs = src.g;

     if (cs <= 0.5) {
         blendValue = cb - (1.0-2.0*cs)*cb*(1.0-cb);
     } else {
         blendValue = cb + (2.0*cs-1.0)*(softlight_d(cb)-cb);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.g = blendValue;

     cb = base.b;
     cs = src.b;

     if (cs <= 0.5) {
         blendValue = cb - (1.0-2.0*cs)*cb*(1.0-cb);
     } else {
         blendValue = cb + (2.0*cs-1.0)*(softlight_d(cb)-cb);
     }

     blendValue = (1.0-alpha_src)*cb + alpha_src*((1.0-alpha_base)*cs + alpha_base*blendValue);

     blendColor.b = blendValue;

     return blendColor;
 }

 vec3 blend_normal(vec3 base, vec3 src, float alpha_base, float alpha_src)
 {
     vec3 blendValue;

     blendValue = src;

     blendValue = (1.0-alpha_src)*base + alpha_src*((1.0-alpha_base)*src + alpha_base*blendValue);

     return blendValue;
 }

 vec3 RGB2HSL(vec3 color)
 {
     float R;
     float G;
     float B;
     float maxV;
     float minV;
     float del_R;
     float del_G;
     float del_B;
     float del_Max;
     float H;
     float S;
     float L;

     R = color.r;
     G = color.g;
     B = color.b;

     minV = min(R, min(G, B));    //Min. value of RGB
     maxV = max(R, max(G, B));    //Max. value of RGB
     del_Max = maxV - minV;        //Delta RGB value

     L = (maxV + minV) / 2.0;

     if (del_Max == 0.0)           //This is a gray, no chroma...
     {
         H = 0.0;                  //HSL results = 0 ÷ 1
         S = 0.0;
     } else {                       //Chromatic data...
         if (L < 0.5) S = del_Max / (maxV + minV);
         else         S = del_Max / (2.0 - maxV - minV);

         del_R = (((maxV - R) / 6.0) + (del_Max / 2.0)) / del_Max;
         del_G = (((maxV - G) / 6.0) + (del_Max / 2.0)) / del_Max;
         del_B = (((maxV - B) / 6.0) + (del_Max / 2.0)) / del_Max;

         if      (R == maxV) H = del_B - del_G;
         else if (G == maxV) H = (1.0 / 3.0) + del_R - del_B;
         else if (B == maxV) H = (2.0 / 3.0) + del_G - del_R;

         if (H < 0.0)  H += 1.0;
         if (H > 1.0)  H -= 1.0;
     }

     return vec3(H, S, L);
 }

 float Hue2RGB(float v1, float v2, float vH)
 {
     if (vH < 0.0) vH += 1.0;
     if (vH > 1.0) vH -= 1.0;
     if (6.0 * vH < 1.0) return v1 + (v2 - v1) * 6.0 * vH;
     if (2.0 * vH < 1.0) return v2;
     if (3.0 * vH < 2.0) return v1 + (v2 - v1) * ((2.0 / 3.0) - vH) * 6.0;

     return v1;
 }

 vec3 HSL2RGB(vec3 hsl)
 {
     float R;
     float G;
     float B;
     float var_1;
     float var_2;
     float H = hsl.x;
     float S = hsl.y;
     float L = hsl.z;

     if (S == 0.0)                       //HSL values = 0 ÷ 1
     {
         R = L;
         G = L;
         B = L;
     } else {
         if (L < 0.5) var_2 = L * (1.0 + S);
         else         var_2 = (L + S) - (S * L);

         var_1 = 2.0 * L - var_2;

         R = Hue2RGB(var_1, var_2, H + (1.0 / 3.0));
         G = Hue2RGB(var_1, var_2, H);
         B = Hue2RGB(var_1, var_2, H - (1.0 / 3.0));
     }
     return vec3(R,G,B);
 }

 float map_hue(int range, float hue_offset[7], float h)
 {
     float h_r = h + (hue_offset[0] + hue_offset[range])*0.5;

     if (h_r < 0.0)
         h_r = h_r + 1.0;
     else if (h_r > 1.0)
         h_r = h_r -1.0;

     return h_r;
 }

 float map_lightness(int range, float lightness_offset[7], float l)
 {
     float l_off = (lightness_offset[0] + lightness_offset[range])*0.5;
     float l_r = l;

     if (l_off < 0.0)
         l_r = l*(l_off+1.0);
     else
         l_r = l+(l_off*(1.0-l));

     return l_r;
 }

 float map_saturaion(int range, float saturation_offset[7], float s)
 {
     float s_off = (saturation_offset[0] + saturation_offset[range]);
     float s_r = s*(s_off+1.0);
     s_r = clamp(s_r, 0.0, 1.0);

     return s_r;
 }

 vec3 hueSaturation(vec3 color, vec3 master_offset, vec3 red_offset, vec3 yellow_offset, vec3 green_offset, vec3 cyan_offset, vec3 blue_offset, vec3 megenta_offset, float overlap_hue)
 {
     vec3 hsl = RGB2HSL(color);
     float h = hsl.x;
     float s = hsl.y;
     float l = hsl.z;

     float hue_thresholds[7];
     hue_thresholds[0] = 0.5/6.0;
     hue_thresholds[1] = 1.5/6.0;
     hue_thresholds[2] = 2.5/6.0;
     hue_thresholds[3] = 3.5/6.0;
     hue_thresholds[4] = 4.5/6.0;
     hue_thresholds[5] = 5.5/6.0;
     hue_thresholds[6] = 6.5/6.0;


     float hue_offset[7];
     hue_offset[0] = master_offset[0];
     hue_offset[1] = red_offset[0];
     hue_offset[2] = yellow_offset[0];
     hue_offset[3] = green_offset[0];
     hue_offset[4] = cyan_offset[0];
     hue_offset[5] = blue_offset[0];
     hue_offset[6] = megenta_offset[0];

     float lightness_offset[7];
     lightness_offset[0] = master_offset[1];
     lightness_offset[1] = red_offset[1];
     lightness_offset[2] = yellow_offset[1];
     lightness_offset[3] = green_offset[1];
     lightness_offset[4] = cyan_offset[1];
     lightness_offset[5] = blue_offset[1];
     lightness_offset[6] = megenta_offset[1];

     float saturation_offset[7];
     saturation_offset[0] = master_offset[2];
     saturation_offset[1] = red_offset[2];
     saturation_offset[2] = yellow_offset[2];
     saturation_offset[3] = green_offset[2];
     saturation_offset[4] = cyan_offset[2];
     saturation_offset[5] = blue_offset[2];
     saturation_offset[6] = megenta_offset[2];

     int use_secondary_hue = 0;
     int hue = 0;
     int secondary_hue = 0;
     float primary_intensity   = 0.0;
     float secondary_intensity = 0.0;

     float h_r = h;
     float l_r = l;
     float s_r = s;

     for (int hue_counter=0; hue_counter<7; hue_counter++) {
         if (h < hue_thresholds[hue_counter]+overlap_hue) {
             float hue_threshold = hue_thresholds[hue_counter];
             hue = hue_counter;

             if (overlap_hue>0.0 && h>hue_threshold-overlap_hue) {
                 secondary_hue = hue_counter + 1;
                 use_secondary_hue = 1;
                 secondary_intensity = (h - hue_threshold + overlap_hue) / (2.0 * overlap_hue);
                 primary_intensity = 1.0 - secondary_intensity;
             } else {
                 use_secondary_hue = 0;
             }

             break;
         }
     }

     if (hue>=6) {
         hue = 0;
         use_secondary_hue = 0;
     }

     if (secondary_hue>=6)
         secondary_hue = 0;

     hue = hue + 1;
     secondary_hue =  secondary_hue + 1;

     if (use_secondary_hue == 1) {
         float h_primary = map_hue(hue, hue_offset, h);
         float h_secondary = map_hue(secondary_hue, hue_offset, h);

         // Find nearest hue on the circle between primary and secondary hue
         float diff =  h_primary - h_secondary;
         if(diff<-0.5)
             h_secondary = h_secondary - 1.0;
         else if (diff>=0.5)
             h_secondary = h_secondary + 1.0;

         h_r = h_primary * primary_intensity + h_secondary * secondary_intensity;
         l_r = map_lightness(hue, lightness_offset, l) * primary_intensity + map_lightness(secondary_hue, lightness_offset, l) * secondary_intensity;
         s_r = map_saturaion(hue, saturation_offset, s) * primary_intensity + map_saturaion(secondary_hue, saturation_offset, s) * secondary_intensity;
     } else {
         h_r = map_hue(hue, hue_offset, h);
         l_r = map_lightness(hue, lightness_offset, l);
         s_r = map_saturaion(hue, saturation_offset, s);
     }

     return HSL2RGB(vec3(h_r, s_r, l_r));
 }

 void main()
 {
     vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);

     // curve
     vec3 curveColor;
     curveColor.r = texture2D(inputImageTexture2, vec2(textureColor.r, 0.0)).r;
     curveColor.g = texture2D(inputImageTexture2, vec2(textureColor.g, 0.0)).g;
     curveColor.b = texture2D(inputImageTexture2, vec2(textureColor.b, 0.0)).b;

     // mask
     vec3 maskColor = mix(vec3(0.0, 0.0, 0.0), curveColor, 1.0-textureCoordinate.y);
     maskColor = blend_softlight(curveColor, maskColor, 1.0, 1.0);

     // hue/saturation
     vec3 hslColor = maskColor;
     vec3 masterOff = vec3(0.0/180.0, 12.0/100.0, 6.0/100.0);
     vec3 redOff = vec3(0.0);
     vec3 yellowOff = vec3(0.0);
     vec3 greenOff = vec3(0.0);
     vec3 cyanOff = vec3(0.0);
     vec3 blueOff = vec3(0.0);
     vec3 megentaOff = vec3(0.0);
     hslColor = hueSaturation(maskColor, masterOff, redOff, yellowOff, greenOff, cyanOff, blueOff, megentaOff, 30.0/100.0*21.0/100.0);

     // brightness/saturation
     vec3 brightnessColor;
     brightnessColor.r = texture2D(inputImageTexture3, vec2(hslColor.r, 0.0)).r;
     brightnessColor.g = texture2D(inputImageTexture3, vec2(hslColor.g, 0.0)).g;
     brightnessColor.b = texture2D(inputImageTexture3, vec2(hslColor.b, 0.0)).b;

     // screen
     vec3 screenColor;
     vec3 startRed = vec3(1.0, 10.0/255.0, 0.0);
     vec3 endRed = vec3(247.0/255.0, 75.0/255.0, 0.0);
     screenColor = blend_screen(brightnessColor, mix(startRed, endRed, textureCoordinate.x), 1.0, 0.14);

     gl_FragColor = vec4(mix(textureColor.rgb, screenColor.rgb, p1), 1.0);
 }