precision highp float;

 varying highp vec2 textureCoordinate;

 uniform sampler2D inputImageTexture;
 uniform sampler2D inputImageTexture2; //curve

 uniform float p1;

 float sqrtX(float x)
 {
     float r = 0.0;

     if(x <= 0.25) {
         r = ((16.0*x - 12.0)*x + 4.0) * x;
     } else {
         r = sqrt(x);
     }

     return r;
 }

 float blend(float base, float src)
 {
     float r = 0.0;

     if(src <= 0.5) {
         r = base - (1.0 - 2.0*src) * base * (1.0 - base);
     } else {
         r = base + (2.0*src - 1.0) * (sqrtX(base) - base);
     }

     return r;
 }

 float blenWithAlpha(float base, float src, float baseAplha, float srcAlpha)
 {
     float r = 0.0;

     r = (1.0 - srcAlpha)*base + srcAlpha*((1.0-baseAplha)*src + baseAplha*blend(base, src));

     return r;
 }

 void main()
 {
     vec4 color = texture2D(inputImageTexture, textureCoordinate);;

     //curve
     vec4 mapped = color;
     mapped.r = texture2D(inputImageTexture2, vec2(color.r, 0.0)).r;
     mapped.g = texture2D(inputImageTexture2, vec2(color.g, 0.0)).g;
     mapped.b = texture2D(inputImageTexture2, vec2(color.b, 0.0)).b;

     //purple overlay
     vec4 purple = vec4(0.81, 0.11, 0.65, 0.3);

     vec4 base = mapped;
     vec4 overlay = purple;

     //
     //     float alphaDivisor = base.a + step(base.a, 0.0); // Protect against a divide-by-zero blacking out things in the output
     //     mapped = base * (overlay.a * (base / alphaDivisor) + (2.0 * overlay * (1.0 - (base / alphaDivisor)))) + overlay * (1.0 - base.a) + base * (1.0 - overlay.a);

     mapped.r = blenWithAlpha(base.r, overlay.r, base.a, overlay.a);
     mapped.g = blenWithAlpha(base.g, overlay.g, base.a, overlay.a);
     mapped.b = blenWithAlpha(base.b, overlay.b, base.a, overlay.a);

     mapped.a = 1.0;

     //strength
     mapped.rgb = mix(color.rgb, mapped.rgb, p1);

     gl_FragColor = mapped;
 }