precision highp float;

 varying highp vec2 textureCoordinate;

 uniform sampler2D inputImageTexture;
 uniform sampler2D inputImageTexture2; //curve

 uniform float p1;

 uniform float inputImageTextureHeight;
 uniform float inputImageTextureWidth;

 vec4 sharpenImage()
 {
     vec2 leftTextureCoordinate;
     vec2 rightTextureCoordinate;

     vec2 topTextureCoordinate;
     vec2 topLeftTextureCoordinate;
     vec2 topRightTextureCoordinate;

     vec2 bottomTextureCoordinate;
     vec2 bottomLeftTextureCoordinate;
     vec2 bottomRightTextureCoordinate;

     mat3 convolutionMatrix = mat3(0.0751, 0.1238, 0.0751, 0.1238, 0.2042, 0.1238, 0.0751, 0.1238, 0.0751);

     float texelWidth = 1.0 / inputImageTextureWidth;
     float texelHeight = 1.0 / inputImageTextureHeight;

     vec2 widthStep = vec2(texelWidth, 0.0);
     vec2 heightStep = vec2(0.0, texelHeight);
     vec2 widthHeightStep = vec2(texelWidth, texelHeight);
     vec2 widthNegativeHeightStep = vec2(texelWidth, -texelHeight);

     leftTextureCoordinate = textureCoordinate.xy - widthStep;
     rightTextureCoordinate = textureCoordinate.xy + widthStep;

     topTextureCoordinate = textureCoordinate.xy - heightStep;
     topLeftTextureCoordinate = textureCoordinate.xy - widthHeightStep;
     topRightTextureCoordinate = textureCoordinate.xy + widthNegativeHeightStep;

     bottomTextureCoordinate = textureCoordinate.xy + heightStep;
     bottomLeftTextureCoordinate = textureCoordinate.xy - widthNegativeHeightStep;
     bottomRightTextureCoordinate = textureCoordinate.xy + widthHeightStep;

     vec3 bottomColor = texture2D(inputImageTexture, bottomTextureCoordinate).rgb;
     vec3 bottomLeftColor = texture2D(inputImageTexture, bottomLeftTextureCoordinate).rgb;
     vec3 bottomRightColor = texture2D(inputImageTexture, bottomRightTextureCoordinate).rgb;
     vec4 centerColor = texture2D(inputImageTexture, textureCoordinate);
     vec3 leftColor = texture2D(inputImageTexture, leftTextureCoordinate).rgb;
     vec3 rightColor = texture2D(inputImageTexture, rightTextureCoordinate).rgb;
     vec3 topColor = texture2D(inputImageTexture, topTextureCoordinate).rgb;
     vec3 topRightColor = texture2D(inputImageTexture, topRightTextureCoordinate).rgb;
     vec3 topLeftColor = texture2D(inputImageTexture, topLeftTextureCoordinate).rgb;

     vec3 blurColor = topLeftColor * convolutionMatrix[0][0] + topColor * convolutionMatrix[0][1] + topRightColor * convolutionMatrix[0][2];
     blurColor += leftColor * convolutionMatrix[1][0] + centerColor.rgb * convolutionMatrix[1][1] + rightColor * convolutionMatrix[1][2];
     blurColor += bottomLeftColor * convolutionMatrix[2][0] + bottomColor * convolutionMatrix[2][1] + bottomRightColor * convolutionMatrix[2][2];

     float gaussianWeightTotal = convolutionMatrix[0][0] + convolutionMatrix[0][1] + convolutionMatrix[0][2] +
     convolutionMatrix[1][0] + convolutionMatrix[1][1] + convolutionMatrix[1][2] +
     convolutionMatrix[2][0] + convolutionMatrix[2][1] + convolutionMatrix[2][2];
     blurColor = blurColor / gaussianWeightTotal;

     vec3 sharpColor = 1.4 * centerColor.rgb - 0.4 * blurColor;

     return vec4(sharpColor, centerColor.a);
 }

 float sqrtX(float x)
 {
     float r = 0.0;

     if(x <= 0.25) {
         r = ((16.0*x - 12.0)*x + 4.0) * x;
     } else {
         r = sqrt(x);
     }

     return r;
 }

 float blend(float base, float src)
 {
     float r = 0.0;

     if(src <= 0.5) {
         r = base - (1.0 - 2.0*src) * base * (1.0 - base);
     } else {
         r = base + (2.0*src - 1.0) * (sqrtX(base) - base);
     }

     return r;
 }

 float blenWithAlpha(float base, float src, float baseAplha, float srcAlpha)
 {
     float r = 0.0;

     r = (1.0 - srcAlpha)*base + srcAlpha*((1.0-baseAplha)*src + baseAplha*blend(base, src));

     return r;
 }

 void main()
 {
     vec4 color = sharpenImage();

     //curve
     vec4 mapped = color;
     mapped.r = texture2D(inputImageTexture2, vec2(color.r, 0.0)).r;
     mapped.g = texture2D(inputImageTexture2, vec2(color.g, 0.0)).g;
     mapped.b = texture2D(inputImageTexture2, vec2(color.b, 0.0)).b;

     //purple overlay
     vec4 purple = vec4(0.81, 0.11, 0.65, 0.3);

     vec4 base = mapped;
     vec4 overlay = purple;

     //
     //     float alphaDivisor = base.a + step(base.a, 0.0); // Protect against a divide-by-zero blacking out things in the output
     //     mapped = base * (overlay.a * (base / alphaDivisor) + (2.0 * overlay * (1.0 - (base / alphaDivisor)))) + overlay * (1.0 - base.a) + base * (1.0 - overlay.a);

     mapped.r = blenWithAlpha(base.r, overlay.r, base.a, overlay.a);
     mapped.g = blenWithAlpha(base.g, overlay.g, base.a, overlay.a);
     mapped.b = blenWithAlpha(base.b, overlay.b, base.a, overlay.a);

     mapped.a = 1.0;

     //strength
     mapped.rgb = mix(color.rgb, mapped.rgb, p1);

     gl_FragColor = mapped;
 }